@@ -37,6 +37,13 @@ if [[ -d $CLOUDKITTY_DIR/bin ]]; then
 else
     CLOUDKITTY_BIN_DIR=$(get_python_exec_prefix)
 fi
+CLOUDKITTY_UWSGI=cloudkitty.wsgi.api:application
+if [ ${CLOUDKITTY_USE_UWSGI,,} == 'true' ]; then
+    CLOUDKITTY_ENDPOINT=$CLOUDKITTY_SERVICE_PROTOCOL://$CLOUDKITTY_SERVICE_HOST/rating
+else
+    CLOUDKITTY_ENDPOINT=$CLOUDKITTY_SERVICE_PROTOCOL://$CLOUDKITTY_SERVICE_HOSTPORT
+fi
+
 
 # Functions
 # ---------
@@ -51,10 +58,7 @@ function create_cloudkitty_accounts {
     local cloudkitty_service=$(get_or_create_service "cloudkitty" \
         "rating" "OpenStack Rating")
     get_or_create_endpoint $cloudkitty_service \
-        "$REGION_NAME" \
-        "$CLOUDKITTY_SERVICE_PROTOCOL://$CLOUDKITTY_SERVICE_HOSTPORT/" \
-        "$CLOUDKITTY_SERVICE_PROTOCOL://$CLOUDKITTY_SERVICE_HOSTPORT/" \
-        "$CLOUDKITTY_SERVICE_PROTOCOL://$CLOUDKITTY_SERVICE_HOSTPORT/"
+        "$REGION_NAME" $CLOUDKITTY_ENDPOINT
 
     # Create the rating role
     get_or_create_role "rating"
@@ -73,53 +77,21 @@ function is_cloudkitty_enabled {
     return 1
 }
 
-# Remove WSGI files, disable and remove Apache vhost file
-function _cloudkitty_cleanup_apache_wsgi {
-    if is_service_enabled ck-api && [ "$CLOUDKITTY_USE_MOD_WSGI" == "True" ]; then
-        sudo rm -f "$CLOUDKITTY_WSGI_DIR"/*
-        sudo rm -rf  "$CLOUDKITTY_WSGI_DIR"
-        sudo rm -f $(apache_site_config_for cloudkitty)
-    fi
-}
-
 # cleanup_cloudkitty() - Remove residual data files, anything left over from previous
 # runs that a clean run would need to clean up
 function cleanup_cloudkitty {
-    _cloudkitty_cleanup_apache_wsgi
     # Clean up dirs
-    rm -rf $CLOUDKITTY_AUTH_CACHE_DIR/*
     rm -rf $CLOUDKITTY_CONF_DIR/*
     rm -rf $CLOUDKITTY_OUTPUT_BASEPATH/*
     for i in $(find $CLOUDKITTY_ENABLED_DIR -iname '_[0-9]*.py' -printf '%f\n'); do
         rm -f "${CLOUDKITTY_HORIZON_ENABLED_DIR}/$i"
     done
-}
-
-
-# Configure mod_wsgi
-function _cloudkitty_config_apache_wsgi {
-    sudo mkdir -m 755 -p $CLOUDKITTY_WSGI_DIR
-
-    local cloudkitty_apache_conf=$(apache_site_config_for cloudkitty)
-    local venv_path=""
-
-    # Copy proxy vhost and wsgi file
-    sudo cp $CLOUDKITTY_DIR/cloudkitty/api/app.wsgi $CLOUDKITTY_WSGI_DIR/app.wsgi
-
-    if [[ ${USE_VENV} = True ]]; then
-        venv_path="python-path=${PROJECT_VENV["cloudkitty"]}/lib/$(python_version)/site-packages"
+    if [ ${CLOUDKITTY_USE_UWSGI,,} == 'true' ]; then
+        remove_uwsgi_config "$CLOUDKITTY_UWSGI_CONF"
     fi
-
-    sudo cp $CLOUDKITTY_DIR/devstack/apache-cloudkitty.template $cloudkitty_apache_conf
-    sudo sed -e "
-        s|%PORT%|$CLOUDKITTY_SERVICE_PORT|g;
-        s|%APACHE_NAME%|$APACHE_NAME|g;
-        s|%WSGIAPP%|$CLOUDKITTY_WSGI_DIR/app.wsgi|g;
-        s|%USER%|$STACK_USER|g;
-        s|%VIRTUALENV%|$DEVSTACK_VENV|g
-    " -i $cloudkitty_apache_conf
 }
 
+
 # configure_cloudkitty() - Set config files, create data dirs, etc
 function configure_cloudkitty {
     setup_develop $CLOUDKITTY_DIR
@@ -165,7 +137,7 @@ function configure_cloudkitty {
         iniset $CLOUDKITTY_CONF fetcher_keystone keystone_version 3
     fi
 
-    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "influxdb" ]; then
+    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "influxdb" ] && [ "$CLOUDKITTY_INFLUX_VERSION" == 1 ]; then
         iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} user ${CLOUDKITTY_INFLUXDB_USER}
         iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} password ${CLOUDKITTY_INFLUXDB_PASSWORD}
         iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} database ${CLOUDKITTY_INFLUXDB_DATABASE}
@@ -173,6 +145,14 @@ function configure_cloudkitty {
         iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} port ${CLOUDKITTY_INFLUXDB_PORT}
     fi
 
+    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "influxdb" ] && [ "$CLOUDKITTY_INFLUX_VERSION" == 2 ]; then
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} host ${CLOUDKITTY_INFLUXDB_HOST}
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} port ${CLOUDKITTY_INFLUXDB_PORT}
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} url "http://${CLOUDKITTY_INFLUXDB_HOST}:${CLOUDKITTY_INFLUXDB_PORT}"
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} token ${CLOUDKITTY_INFLUXDB_PASSWORD}
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} version 2
+    fi
+
     if [ "$CLOUDKITTY_STORAGE_BACKEND" == "elasticsearch" ]; then
         iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} host ${CLOUDKITTY_ELASTICSEARCH_HOST}
         iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} index_name ${CLOUDKITTY_ELASTICSEARCH_INDEX}
@@ -207,8 +187,8 @@ function configure_cloudkitty {
     # keystone middleware
     configure_keystone_authtoken_middleware $CLOUDKITTY_CONF cloudkitty
 
-    if is_service_enabled ck-api && [ "$CLOUDKITTY_USE_MOD_WSGI" == "True" ]; then
-        _cloudkitty_config_apache_wsgi
+    if [ ${CLOUDKITTY_USE_UWSGI,,} == 'true' ]; then
+        write_uwsgi_config "$CLOUDKITTY_UWSGI_CONF" "$CLOUDKITTY_UWSGI" "/rating" "" "cloudkitty"
     fi
 }
 
@@ -219,17 +199,6 @@ function wait_for_gnocchi() {
     fi
 }
 
-# create_cloudkitty_cache_dir() - Part of the init_cloudkitty() process
-function create_cloudkitty_cache_dir {
-    # Create cache dir
-    sudo mkdir -p $CLOUDKITTY_AUTH_CACHE_DIR/api
-    sudo chown $STACK_USER $CLOUDKITTY_AUTH_CACHE_DIR/api
-    rm -f $CLOUDKITTY_AUTH_CACHE_DIR/api/*
-    sudo mkdir -p $CLOUDKITTY_AUTH_CACHE_DIR/registry
-    sudo chown $STACK_USER $CLOUDKITTY_AUTH_CACHE_DIR/registry
-    rm -f $CLOUDKITTY_AUTH_CACHE_DIR/registry/*
-}
-
 # create_cloudkitty_data_dir() - Part of the init_cloudkitty() process
 function create_cloudkitty_data_dir {
     # Create data dir
@@ -242,9 +211,13 @@ function create_cloudkitty_data_dir {
 }
 
 function create_influxdb_database {
-    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "influxdb" ]; then
+    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "influxdb" ] && [ "$CLOUDKITTY_INFLUX_VERSION" == 1 ]; then
         influx -execute "CREATE DATABASE ${CLOUDKITTY_INFLUXDB_DATABASE}"
     fi
+    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "influxdb" ] && [ "$CLOUDKITTY_INFLUX_VERSION" == 2 ]; then
+        influx setup --username ${CLOUDKITTY_INFLUXDB_USER} --password ${CLOUDKITTY_INFLUXDB_PASSWORD} --token ${CLOUDKITTY_INFLUXDB_PASSWORD} --org openstack --bucket cloudkitty --force
+    fi
+
 }
 
 function create_elasticsearch_index {
@@ -261,11 +234,6 @@ function create_opensearch_index {
 
 # init_cloudkitty() - Initialize CloudKitty database
 function init_cloudkitty {
-    # Delete existing cache
-    sudo rm -rf $CLOUDKITTY_AUTH_CACHE_DIR
-    sudo mkdir -p $CLOUDKITTY_AUTH_CACHE_DIR
-    sudo chown $STACK_USER $CLOUDKITTY_AUTH_CACHE_DIR
-
     # Delete existing cache
     sudo rm -rf $CLOUDKITTY_OUTPUT_BASEPATH
     sudo mkdir -p $CLOUDKITTY_OUTPUT_BASEPATH
@@ -287,7 +255,6 @@ function init_cloudkitty {
     fi
     $CLOUDKITTY_BIN_DIR/cloudkitty-storage-init
 
-    create_cloudkitty_cache_dir
     create_cloudkitty_data_dir
 }
 
@@ -296,11 +263,27 @@ function install_influx_ubuntu {
     sudo dpkg -i --skip-same-version ${influxdb_file}
 }
 
+function install_influx_v2_ubuntu {
+    local influxdb_file=$(get_extra_file https://dl.influxdata.com/influxdb/releases/influxdb2_2.7.5-1_amd64.deb)
+    sudo dpkg -i --skip-same-version ${influxdb_file}
+    local influxcli_file=$(get_extra_file https://dl.influxdata.com/influxdb/releases/influxdb2-client-2.7.3-linux-amd64.tar.gz)
+    tar xvzf ${influxcli_file}
+    sudo cp ./influx /usr/local/bin/
+}
+
 function install_influx_fedora {
     local influxdb_file=$(get_extra_file https://dl.influxdata.com/influxdb/releases/influxdb-1.6.3.x86_64.rpm)
     sudo yum localinstall -y ${influxdb_file}
 }
 
+function install_influx_v2_fedora {
+    local influxdb_file=$(get_extra_file https://dl.influxdata.com/influxdb/releases/influxdb2-2.7.5-1.x86_64.rpm)
+    sudo yum localinstall -y ${influxdb_file}
+    local influxcli_file=$(get_extra_file https://dl.influxdata.com/influxdb/releases/influxdb2-client-2.7.3-linux-amd64.tar.gz)
+    tar xvzf ${influxcli_file}
+    sudo cp ./influx /usr/local/bin/
+}
+
 function install_influx {
     if is_ubuntu; then
         install_influx_ubuntu
@@ -313,6 +296,19 @@ function install_influx {
     sudo systemctl start influxdb || sudo systemctl restart influxdb
 }
 
+
+function install_influx_v2 {
+    if is_ubuntu; then
+        install_influx_v2_ubuntu
+    elif is_fedora; then
+        install_influx_v2_fedora
+    else
+        die $LINENO "Distribution must be Debian or Fedora-based"
+    fi
+    sudo cp -f "${CLOUDKITTY_DIR}"/devstack/files/influxdb.conf /etc/influxdb/influxdb.conf
+    sudo systemctl start influxdb || sudo systemctl restart influxdb
+}
+
 function install_elasticsearch_ubuntu {
     local opensearch_file=$(get_extra_file https://artifacts.opensearch.org/releases/bundle/opensearch/1.3.9/opensearch-1.3.9-linux-x64.deb)
     sudo dpkg -i --skip-same-version ${opensearch_file}
@@ -367,29 +363,30 @@ function install_opensearch {
 function install_cloudkitty {
     git_clone $CLOUDKITTY_REPO $CLOUDKITTY_DIR $CLOUDKITTY_BRANCH
     setup_develop $CLOUDKITTY_DIR
-
-    if [ $CLOUDKITTY_STORAGE_BACKEND == 'influxdb' ]; then
+    if [ $CLOUDKITTY_STORAGE_BACKEND == 'influxdb' ] && [ "$CLOUDKITTY_INFLUX_VERSION" == 1 ]; then
         install_influx
+    elif [ $CLOUDKITTY_STORAGE_BACKEND == 'influxdb' ] && [ "$CLOUDKITTY_INFLUX_VERSION" == 2 ]; then
+        install_influx_v2
     elif [ $CLOUDKITTY_STORAGE_BACKEND == 'elasticsearch' ]; then
         install_elasticsearch
     elif [ $CLOUDKITTY_STORAGE_BACKEND == 'opensearch' ]; then
         install_opensearch
     fi
+    if [ ${CLOUDKITTY_USE_UWSGI,,} == 'true' ]; then
+        pip_install uwsgi
+    fi
 }
 
 # start_cloudkitty() - Start running processes, including screen
 function start_cloudkitty {
     run_process ck-proc "$CLOUDKITTY_BIN_DIR/cloudkitty-processor --config-file=$CLOUDKITTY_CONF"
-    if [[ "$CLOUDKITTY_USE_MOD_WSGI" == "False" ]]; then
+    if [ ${CLOUDKITTY_USE_UWSGI,,} == 'true' ]; then
+        run_process ck-api "$CLOUDKITTY_BIN_DIR/uwsgi --ini $CLOUDKITTY_UWSGI_CONF"
+    else
         run_process ck-api "$CLOUDKITTY_BIN_DIR/cloudkitty-api --host $CLOUDKITTY_SERVICE_HOST --port $CLOUDKITTY_SERVICE_PORT"
-    elif is_service_enabled ck-api; then
-        enable_apache_site cloudkitty
-        echo_summary "Waiting 15s for cloudkitty-processor to authenticate against keystone before apache is restarted."
-        sleep 15s
-        restart_apache_server
     fi
-    echo "Waiting for ck-api ($CLOUDKITTY_SERVICE_HOST:$CLOUDKITTY_SERVICE_PORT) to start..."
-    if ! wait_for_service $SERVICE_TIMEOUT $CLOUDKITTY_SERVICE_PROTOCOL://$CLOUDKITTY_SERVICE_HOST:$CLOUDKITTY_SERVICE_PORT; then
+    echo "Waiting for ck-api ($CLOUDKITTY_SERVICE_HOST) to start..."
+    if ! wait_for_service $SERVICE_TIMEOUT $CLOUDKITTY_ENDPOINT; then
         die $LINENO "ck-api did not start"
     fi
 }
@@ -402,13 +399,8 @@ function stop_cloudkitty {
     fi
 
     if is_service_enabled ck-api ; then
-        if [ "$CLOUDKITTY_USE_MOD_WSGI" == "True" ]; then
-            disable_apache_site cloudkitty
-            restart_apache_server
-        else
-            # Kill the cloudkitty screen windows
-            stop_process ck-api
-        fi
+        # Kill the cloudkitty screen windows
+        stop_process ck-api
     fi
 }
 
@@ -440,7 +432,6 @@ function update_horizon_static {
         $django_admin collectstatic --noinput
     DJANGO_SETTINGS_MODULE=openstack_dashboard.settings \
         $django_admin compress --force
-    restart_apache_server
 }
 
 # Upgrade cloudkitty database
@@ -463,7 +454,6 @@ if is_service_enabled ck-api; then
         if is_service_enabled horizon; then
             install_cloudkitty_dashboard
         fi
-        cleanup_cloudkitty
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         echo_summary "Configuring CloudKitty"
         configure_cloudkitty
@@ -487,6 +477,10 @@ if is_service_enabled ck-api; then
     if [[ "$1" == "unstack" ]]; then
         stop_cloudkitty
     fi
+
+    if [[ "$1" == "clean" ]]; then
+        cleanup_cloudkitty
+    fi
 fi
 
 # Restore xtrace
